/*==================================================================
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corp. 2003.  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation.  This sample code is
 * not part of any standard or IBM product and is provided to you
 * solely for the purpose of assisting you in the development of
 * your applications.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have
 * been advised of the possibility of such damages.
 *===============================================================*/

package contactlist.struts.persistence;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import contactlist.forms.Contact;
import contactlist.struts.utilities.AIMException;
import contactlist.struts.utilities.AIMWrapperException;


/**
 * DB broker implementation 
 * 
 * Creation date: (11/30/2003 10:00:00 PM)
 * @author: Tim Hanis
 */
public class ContactListDBBroker extends AbstractDBBroker {
	
	//  Logger
	protected Log log = null;
	
	/**
	 *  Null Constructor not visible
	 */
	private ContactListDBBroker() {
		super();
	}
	
	public void populate() {
	}
	
	
	/**
	 *  Broker initializer
	 */
	public static void initialize(String datasourceName) {
		ContactListDBBroker broker = new ContactListDBBroker();
		broker.datasourceName = datasourceName;
		broker.log = LogFactory.getLog("contactlist.struts.persistence");
		brokerInstance = broker;
	}
	
	
	/**
	 * Get the Contact information given an Contact number
	 * @param owner java.lang.String
	 * @param contactOID java.lang.String
	 * @return Contact
	 */
	public Contact getContact(String owner, String oid) 
			throws AIMException {
			
		//  Tracing
		if (log.isDebugEnabled())
			log.debug("Getting contact "+oid+" for user "+owner);
				
		Connection connection = null;
		PreparedStatement statement = null;
		ResultSet resultSet = null;
		Contact contact = null;
		String sql = "select * from CONTACTS where OWNER = ? and OID = ?";

		try {
			connection = getConnection();
			statement = connection.prepareStatement(sql);
			statement.setString(1, maxLength(owner, 64));
			statement.setString(2, maxLength(oid, 64));
			resultSet = statement.executeQuery();
			contact = createContact(resultSet);
		} 
		catch (Exception e) {
			throw new AIMWrapperException(e.getLocalizedMessage(), e);
		}
		finally {
			close(connection, statement, resultSet);
		}
		return contact;
		
	}
	
	
	/**
	 * Create a Contact
	 * @param Contact
	 */
	public void saveContact(Contact contact) 
			throws AIMException {
			
		//  Tracing
		if (log.isDebugEnabled())
			log.debug("Saving contact for user "+contact.getOwner());

		Connection connection = null;
		PreparedStatement statement = null;
		String sql = "insert into contacts (OWNER, OID, FIRST_NAME, LAST_NAME, EMAIL, TITLE, "+
			"COMPANY, MOBILE_PHONE, BUS_PHONE, FAX_PHONE, WEB, ADDRESS1, ADDRESS2, "+
			"CITY, STATE, ZIP, COUNTRY) VALUES ( ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ? )";
		

		try {
			connection = getConnection();
			statement = connection.prepareStatement(sql);
      		statement.setString(1, maxLength(contact.getOwner(), 64));
      		statement.setString(2, maxLength(contact.getOid(), 64));
      		statement.setString(3, maxLength(contact.getFirstName(), 32));
      		statement.setString(4, maxLength(contact.getLastName(), 32));
     		statement.setString(5, maxLength(contact.getEmail(), 128));
     		statement.setString(6, maxLength(contact.getTitle(), 64));
     		statement.setString(7, maxLength(contact.getCompany(), 64));
      		statement.setString(8, maxLength(contact.getMobilePhone(), 32));
      		statement.setString(9, maxLength(contact.getBusinessPhone(), 32));
      		statement.setString(10, maxLength(contact.getFaxPhone(), 32));
      		statement.setString(11, maxLength(contact.getWeb(), 128));
      		statement.setString(12, maxLength(contact.getAddress(), 32));
      		statement.setString(14, maxLength(contact.getCity(), 32));
      		statement.setString(15, maxLength(contact.getState(), 2));
      		statement.setString(16, maxLength(contact.getZip(), 10));
      		statement.setString(17, maxLength(contact.getCountry(), 32));
			statement.executeUpdate();
		} 
		catch (AIMException e) {
			throw e;
		}
		catch (Exception e) {
			throw new AIMWrapperException(e.getLocalizedMessage(), e);
		}
		finally {
			close(connection, statement, null);
		}
		
	}
	
	
	/**
	 * Delete a Contact
	 * @param Contact
	 */
	public void deleteContact(String owner, String oid) 
			throws AIMException {
			
		//  Tracing
		if (log.isDebugEnabled())
			log.debug("Deleting contact "+oid+" for user "+owner);

		Connection connection = null;
		PreparedStatement statement = null;
		String sql = "delete from CONTACTS where OWNER = ? and OID = ?";

		try {
			connection = getConnection();
			statement = connection.prepareStatement(sql);
			statement.setString(1, owner);
			statement.setString(2, oid);
			statement.executeUpdate();
		} 
		catch (AIMException e) {
			throw e;
		}
		catch (Exception e) {
			throw new AIMWrapperException(e.getLocalizedMessage(), e);
		}
		finally {
			close(connection, statement, null);
		}
		
	}
	
	
	/**
	 * Get the contact list. 
	 * @param filter java.lang.String
	 * @return ContactList
	 */
	public List getContactList(String owner) 
			throws AIMException {
			
		//  Tracing
		if (log.isDebugEnabled())
			log.debug("Getting contact list for user "+owner);

		Connection connection = null;
		PreparedStatement statement = null;
		ResultSet resultSet = null;
		List contactList = new ArrayList();
		String sql = "select OWNER, OID, FIRST_NAME, LAST_NAME, EMAIL, COMPANY, MOBILE_PHONE  from CONTACTS where OWNER = ? order by FIRST_NAME";

		try {
			connection = getConnection();
			statement = connection.prepareStatement(sql);
			statement.setString(1, maxLength(owner, 64));
			resultSet = statement.executeQuery();
			contactList = createContactList(resultSet);
		} 
		catch (Exception e) {
			throw new AIMWrapperException(e.getLocalizedMessage(), e);
		}
		finally {
			close(connection, statement, resultSet);
		}
		return contactList;
		
	}


	/**
	 * Create an instance of this type with the data values
	 * in the given result set.  The result set contains data
	 * to fully initialize this instance. The column names in
	 * the result set are fixed.
	 * @param resultSet java.sql.ResultSet
	 */
	public Contact createContact(ResultSet resultSet) 
			throws AIMException {
				
		Contact contact = new Contact();
		if (resultSet == null)
			return null;
 		try {
			if (resultSet.next()) {		
				contact.setOwner(resultSet.getString("OWNER"));
				contact.setOid(resultSet.getString("OID"));
				contact.setFirstName(resultSet.getString("FIRST_NAME"));
				contact.setLastName(resultSet.getString("LAST_NAME"));
				contact.setEmail(resultSet.getString("EMAIL"));
				contact.setTitle(resultSet.getString("TITLE"));
				contact.setCompany(resultSet.getString("COMPANY"));
				contact.setBusinessPhone(resultSet.getString("BUS_PHONE"));
				contact.setMobilePhone(resultSet.getString("MOBILE_PHONE"));
				contact.setFaxPhone(resultSet.getString("FAX_PHONE"));
				contact.setWeb(resultSet.getString("WEB"));
				contact.setAddress(resultSet.getString("ADDRESS"));
				contact.setCity(resultSet.getString("CITY"));
				contact.setState(resultSet.getString("STATE"));
				contact.setZip(resultSet.getString("ZIP"));
				contact.setCountry(resultSet.getString("COUNTRY"));
			}
		} catch (SQLException e) {
			throw new AIMWrapperException(e.getLocalizedMessage(), e);
		}
		return contact;
	
	}


	/**
	 * Create a list of contacys.  Partial initialization of 
	 * contact elements to display in list.
	 * @param resultSet java.sql.ResultSet
	 */
	public List createContactList(ResultSet resultSet) 
			throws AIMException {
				
		if (resultSet == null)
			return null;
		List contactList = new ArrayList();
			
 		try {
			while (resultSet.next()) {
				Contact contact = new Contact();
				contact.setOwner(resultSet.getString("OWNER"));
				contact.setOid(resultSet.getString("OID"));
				contact.setFirstName(resultSet.getString("FIRST_NAME"));
				contact.setLastName(resultSet.getString("LAST_NAME"));
				contact.setEmail(resultSet.getString("EMAIL"));
				contact.setCompany(resultSet.getString("COMPANY"));
				contact.setMobilePhone(resultSet.getString("MOBILE_PHONE"));
				contactList.add(contact);
			}
		} catch (SQLException e) {
			throw new AIMWrapperException(e.getLocalizedMessage(), e);
		}
		return contactList;
	
	}
	
	
	/**
	 * Test the connection and the required table schema
	 * Throws exceptions on failure
	 */
	public void verifyTables() throws AIMException {

		verifyTableSchema("CONTACTS", "OWNER, OID, FIRST_NAME, LAST_NAME, EMAIL, TITLE, COMPANY, MOBILE_PHONE, "+
			"BUS_PHONE, FAX_PHONE, WEB, ADDRESS1, ADDRESS2, CITY, STATE, ZIP, COUNTRY");
	
	}
	
	
	/**
	 * Test the table schema
	 * Throws exception on failure
	 */
	public void verifyTableSchema(String table, String columns) 
			throws AIMException {
			
		//  Tracing
		if (log.isDebugEnabled())
			log.debug("Verifying db table schema for table "+table);

		Connection connection = null;
		Statement statement = null;
		String sql = "select " + columns + " from " + table + " where 1=0";

		try {
			connection = getConnection();
			statement = connection.createStatement();
			statement.execute(sql);
		} 
		catch (AIMException e) {
			throw e;
		}
		catch (Exception e) {
			throw new AIMWrapperException(e.getLocalizedMessage(), e);
		}
		finally {
			close(connection, statement, null);
		}
		return
		;
	}
	
	/*
	 *  Trim the string to the max length specified
	 */
	 protected String maxLength(String aString, int length) {
	 	if (aString.length() <= length)
	 		return aString;
	 	return aString.substring(0, length);
	 }
	 
}